#ifndef __ADETHERNET_H__
#define __ADETHERNET_H__

#include <stdio.h>
#include <libadwin/bits.h>
#include "adwin_tcp.h"

// Parameter, die zu einem ADwin-System gehoeren (ADEthernet.c kennt nur 1 Device pro Thread gleichzeitig)
enum adwin_tcp_status { ADWIN_TCP_OK, ADWIN_TCP_ERROR, ADWIN_UDP };

typedef struct {
    int sockfd;
    char ip_address[200];
    int32_t port;
    struct sockaddr_in client, server;
    unsigned int timeout;
    char passwd[10];
    int hostlink;
    int32_t fehlercode;
    uint32_t udp_paket_count;
    int32_t ip_version;
    int32_t supports_tcp;
    int32_t supports_udp;
    enum adwin_tcp_status tcp_status;
    adwin_tcp_info_t ati;
    time_t last_access;
} ADwinDevice;


#ifdef BUILD_LIBRARY
    #define NAME(x) Generic_Driver_##x
#else
    #define NAME(x) x
#endif

#ifdef __cplusplus
extern "C" {
#endif

extern int     openConnect (const char *IP, const char *passwd,
                            int link, int Port, unsigned int Timeout);
extern void    closeConnect (void);
extern int     reConnect (const char *IP, const char *passwd,
                          int link, int Port, unsigned int Timeout);

extern void    NAME (Boot) (const char *filename);
extern int32_t NAME (Test_Version) ();
extern processor_type_t NAME (Processor_Type) ();
extern int32_t NAME (Workload) (int32_t priority);
extern int32_t NAME (Free_Mem) (mem_spec_t mem_spec);

extern void    NAME (Load_Process) (const char *filename);
extern void    NAME (Start_Process) (int32_t process_no);
extern void    NAME (Stop_Process) (int32_t process_no);
extern void    NAME (Clear_Process) (int32_t process_no);
extern int32_t NAME (Process_Status) (int32_t process_no);
extern void    NAME (Set_ProcessDelay) (int32_t process_no, int32_t value);
extern int32_t NAME (Get_ProcessDelay) (int32_t process_no);

extern void    NAME (Set_Par) (int32_t index, int32_t value);
extern int32_t NAME (Get_Par) (int32_t index);
extern void    NAME (Get_Par_Block) (int32_t dest[], int32_t start_index, int32_t count);
extern void    NAME (Get_Par_All) (int32_t dest[]);
extern void    NAME (Set_FPar) (int32_t index, float value);
extern float   NAME (Get_FPar) (int32_t index);
extern void    NAME (Get_FPar_Block) (float dest[], int32_t start_index, int32_t count);
extern void    NAME (Get_FPar_All) (float dest[]);
extern void    NAME (Set_FPar_Double) (int32_t index, double value);
extern double  NAME (Get_FPar_Double) (int32_t index);
extern void    NAME (Get_FPar_Block_Double) (double dest[], int32_t start_index, int32_t count);
extern void    NAME (Get_FPar_All_Double) (double dest[]);

extern int32_t NAME (Data_Length) (int32_t data_no);
extern void    NAME (SetData_Long) (int32_t data_no, int32_t src[], int32_t Startindex, int32_t count);
extern void    NAME (GetData_Long) (int32_t data_no, int32_t dest[], int32_t Startindex, int32_t count);
extern void    NAME (SetData_Float) (int32_t data_no, float src[], int32_t Startindex, int32_t count);
extern void    NAME (GetData_Float) (int32_t data_no, float dest[], int32_t Startindex, int32_t count);

extern int32_t NAME (Fifo_Empty) (int32_t fifo_no);
extern int32_t NAME (Fifo_Full) (int32_t fifo_no);
extern void    NAME (Fifo_Clear) (int32_t fifo_no);
extern void    NAME (SetFifo_Byte) (int32_t fifo_no, int8_t src[], int32_t count);
extern void    NAME (GetFifo_Byte) (int32_t fifo_no, int8_t dest[], int32_t count);
extern void    NAME (SetFifo_Short) (int32_t fifo_no, int16_t src[], int32_t count);
extern void    NAME (GetFifo_Short) (int32_t fifo_no, int16_t dest[], int32_t count);
extern void    NAME (SetFifo_Long) (int32_t fifo_no, int32_t src[], int32_t count);
extern void    NAME (GetFifo_Long) (int32_t fifo_no, int32_t dest[], int32_t count);
extern void    NAME (SetFifo_Float) (int32_t fifo_no, float src[], int32_t count);
extern void    NAME (GetFifo_Float) (int32_t fifo_no, float dest[], int32_t count);
extern void    NAME (SetFifo_Double) (int32_t fifo_no, double src[], int32_t count);
extern void    NAME (GetFifo_Double) (int32_t fifo_no, double dest[], int32_t count);
extern void    NAME (SetFifo_Int64) (int32_t fifo_no, int64_t src[], int32_t count);
extern void    NAME (GetFifo_Int64) (int32_t fifo_no, int64_t dest[], int32_t count);

extern void    NAME (Data2File) (const char *filename, int32_t data_no,
                                 int32_t start_index, int32_t count, int mode);

extern void    NAME (SetData_String) (int32_t data_no, const char *src);
extern int32_t NAME (GetData_String) (int32_t data_no, char *dest, int32_t max_count);
extern int32_t NAME (String_Length) (int32_t data_no);


extern int32_t NAME (Get_Last_Error) ();
// todo: should be extern int32_t NAME (Get_Last_Error_Text) (int32_t Errno);
extern int32_t NAME (Get_Last_Error_Text) (int32_t errorcode, char *text, int32_t textlaenge);

// undocumented
extern int     NAME (Clear_Data) (int32_t data_no);
extern adwin_system_data_type_t NAME (GetDataType) (short data_no);
extern void    NAME (Get_Memory_dsp) (int32_t base_addr, int32_t count, int32_t *dest);
extern void    NAME (Set_Fifo) (int32_t fifo_no, adwin_data_type_t type, void *src, int32_t count);
extern void    NAME (Get_Fifo) (int32_t fifo_no, adwin_data_type_t type, void *dest, int32_t count);
extern void    NAME (Set_Data) (int32_t data_no, void *src, adwin_data_type_t adt, adwin_system_data_type_t asdt, int32_t start_index, int32_t count);
extern void    NAME (Get_Data) (int32_t data_no, void *dest, adwin_data_type_t adt, adwin_system_data_type_t asdt, int32_t start_index, int32_t count);

extern int32_t intern_Get_Version_TcpIp();

int32_t map_status_to_errorcode(int32_t status);

#ifdef __cplusplus
}
#endif

#endif
