#ifndef ADWIN_TCP_H
#define ADWIN_TCP_H

#include <stdio.h>

#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <netinet/in.h>

#include "libadwin/bits.h"

#define SMALL_REQ_SIZE (10*1024)

#define ADWIN_TCP_MAGIC_FLAG_REQUEST  0x42ad4214
#define ADWIN_TCP_MAGIC_FLAG_RESPONSE 0x42ad4253
typedef struct adwin_tcp_header_s {
    uint32_t magic_flag;
    uint32_t req_count;
    int32_t length; // not uint32_t because we compare it to result of readn
    uint32_t command_or_status;
} adwin_tcp_header_t;

#define ADWIN_HTONL 1
#define ADWIN_NTOHL 2
typedef struct adwin_tcp_buffer_s {
    int32_t max;  // size allocated for data
    int32_t used; // size used in data
    int32_t do_convert; // convert <used> 32-bit values using htonl (1) or ntohl(2)
    union {
        void *as_void;
        const void *as_const_void;
        int32_t *as_int32;
        uint32_t *as_uint32;
        float *as_float;
        char *as_char;
    } data;                     // buffer for response
} adwin_tcp_buffer_t;

typedef struct adwin_tcp_info_s {
    struct sockaddr_in server_addr;
    int32_t firmware_version, firmware_version_beta;
    int32_t error_number;
    int32_t processortype;
    int32_t connected;
    int32_t initialized;

    int32_t socket;             // the socket...
    uint32_t timeout_ms;        // timeout sent from dll
    uint32_t timeout_size;      // timeout is checked for chunks of this size
    uint32_t dll_version;       // dll version

    uint32_t req_count;         // network byte order, just sent back to server

    uint32_t command;           // command in host byte order
    int32_t status;             // status code or response
    adwin_tcp_buffer_t req;
    int32_t rx_pending;         // size of pending data (not in any buffer)

    adwin_tcp_buffer_t resp;
} adwin_tcp_info_t;

void adwin_tcp_construct(const char *ip, int16_t port, const char *passwd, int32_t timeout);
void adwin_tcp_destroy();

int32_t tcp_read_response(adwin_tcp_info_t *ati, int32_t min_bytes, adwin_tcp_buffer_t *buf);
int32_t tcp_read_pending(adwin_tcp_info_t *ati, int32_t min_bytes, adwin_tcp_buffer_t *buf);
int32_t tcp_discard_pending(adwin_tcp_info_t *ati);
void init_ati(int32_t command);
int32_t tcp_write_request(adwin_tcp_info_t *ati, adwin_tcp_buffer_t *buf);
int32_t bytes_per_value(adwin_data_type_t adt);
void boot_or_loadprocess(const char *filename, int32_t command);
void adwin_tcp_get_data_or_fifo(adwin_system_data_type_t asdt, int32_t datanumber, void *data, int32_t start, int32_t count, int32_t get_fifo);
void adwin_tcp_set_data_or_fifo(adwin_data_type_t pc_datatype, int32_t datanumber, const void *data, int32_t start, int32_t count, int32_t set_fifo);
void set_last_error(int32_t new_error_number);

int32_t check_tcp_ret(int32_t ret, int32_t ref);
void command_4p4(int32_t to_values[4], int32_t from_values[4]);
int32_t map_error_code(int32_t adwin_error);

void adwin_tcp_connect(void);
void adwin_tcp_boot(const char *filename);
void adwin_tcp_load_process(const char *filename);
void adwin_tcp_start_process(int32_t processnumber);
void adwin_tcp_stop_process(int32_t processnumber);
void adwin_tcp_clear_process(int32_t processnumber);
int32_t adwin_tcp_process_status(int32_t processnumber);
float adwin_tcp_workload(void);
int32_t adwin_tcp_testversion(int32_t values[4]);
int32_t adwin_tcp_free_mem(int32_t mem_spec);
void adwin_tcp_peek(int32_t address, int32_t count, int32_t *pc_array);
int32_t adwin_tcp_get_processdelay(int32_t processnumber);
void adwin_tcp_set_processdelay(int32_t processnumber, int32_t delay);

int32_t adwin_tcp_get_par(int32_t par_no);
float adwin_tcp_get_fpar(int32_t par_no);
double adwin_tcp_get_fpar_double(int32_t par_no);
void adwin_tcp_set_par(int32_t par_no, int32_t value);
void adwin_tcp_set_fpar(int32_t par_no, float value);
void adwin_tcp_set_fpar_double(int32_t par_no, double value);

void adwin_tcp_get_par_block(int32_t start, int32_t count, int32_t *pars);
void adwin_tcp_get_par_all(int32_t *pars);
void adwin_tcp_get_fpar_block(int32_t start, int32_t count, float *fpars);
void adwin_tcp_get_fpar_all(float *fpars);
void adwin_tcp_get_fpar_block_double(int32_t start, int32_t count, double *dpars);
void adwin_tcp_get_fpar_all_double(double *dpars);

adwin_data_type_t adwin_tcp_get_data_type(int32_t datanumber);
int32_t adwin_tcp_get_data_length(int32_t datanumber);
void adwin_tcp_clear_data(int32_t datanumber);
void convert_data(int from_type, int to_type, const void *from_data, void *to_data, int32_t count);

void adwin_tcp_get_data(adwin_system_data_type_t asdt, int32_t datanumber, void *data, int32_t start, int32_t count);
void adwin_tcp_get_data_long(int32_t datanumber, void *data, int32_t start, int32_t count);
void adwin_tcp_get_data_float(int32_t datanumber, void *data, int32_t start, int32_t count);
void adwin_tcp_get_data_double(int32_t datanumber, void *data, int32_t start, int32_t count);

int32_t adwin_tcp_string_length(int32_t datanumber);
void adwin_tcp_get_data_string(int32_t datanumber, char *c_str, int32_t max_count);

void adwin_tcp_set_data(adwin_data_type_t pc_datatype, int32_t datanumber, const void *data, int32_t start, int32_t count);
void adwin_tcp_set_data_long(int32_t datanumber, const void *data, int32_t start, int32_t count);
void adwin_tcp_set_data_float(int32_t datanumber, const void *data, int32_t start, int32_t count);
void adwin_tcp_set_data_double(int32_t datanumber, const void *data, int32_t start, int32_t count);
void adwin_tcp_set_data_string(int32_t datanumber, const char *c_str, int32_t max_count);

void adwin_tcp_get_fifo(adwin_data_type_t pc_datatype, int32_t datanumber, void *data, int32_t count);
void adwin_tcp_get_fifo_long(int32_t datanumber, void *data, int32_t count);
void adwin_tcp_get_fifo_float(int32_t datanumber, void *data, int32_t count);
void adwin_tcp_get_fifo_double(int32_t datanumber, void *data, int32_t count);

void adwin_tcp_set_fifo(adwin_data_type_t pc_datatype, int32_t datanumber, void *data, int32_t count);
void adwin_tcp_set_fifo_long(int32_t datanumber, void *data, int32_t count);
void adwin_tcp_set_fifo_float(int32_t datanumber, void *data, int32_t count);
void adwin_tcp_set_fifo_double(int32_t datanumber, void *data, int32_t count);

int32_t adwin_tcp_fifo_full(int32_t datanumber);
int32_t adwin_tcp_fifo_empty(int32_t datanumber);
void adwin_tcp_fifo_clear(int32_t datanumber);

int32_t adwin_tcp_get_last_error(void);
void adwin_tcp_get_last_error_text(int32_t err_no, char *error_text);
void adwin_tcp_create_data(int32_t datanumber, int32_t as_fifo, int32_t datasize, int32_t datatype, int32_t bank);
void adwin_tcp_get_data_info(int32_t datanumber, int32_t *datatype, int32_t *datasize, int32_t *isfifo, int32_t *fifo_full);

processor_type_t adwin_tcp_get_processor_type();

ssize_t readn(int fd, void *vptr, size_t n);
ssize_t writen(int fd, const void *vptr, size_t n);

#endif // ADWIN_TCP_H

