#ifndef __LIBADWIN_MACROS_H__
#define __LIBADWIN_MACROS_H__

#include <string.h>

/**
 * CAST casts "cnt" elements of type "from_type" at position
 * "from_ptr" to type "to_type" at position "to_ptr". Wear a helmet.
 * TB
 *
 * @param from_type type of data to cast.
 * @param from_ptr pointer to start reading from.
 * @param to_type desired type of cast.
 * @param to_ptr pointer to start writing to.
 * @param cnt number of elements to cast.
 */
#define CAST(from_type, from_ptr, to_type, to_ptr, cnt)         \
    do {                                    \
        unsigned long i;                                \
        from_type *__fptr = (from_type *) (from_ptr);           \
        to_type   *__tptr = (to_type *) (to_ptr);           \
        for (i = 0; i < (cnt); i++)                 \
            *(__tptr)++ = (to_type) *__fptr++;          \
    } while(0)


/**
 * FCAST is similar to CAST, but designed to cast from float values to
 * other values. It maps all NANs to MAXINT or something similar.
 */
#define g_QNAN_MASK 0x7F800000
#define FCAST(from_type, from_ptr, to_type, to_ptr, cnt)        \
    do {                                    \
        unsigned long i;                                \
        from_type *__fptr = (from_type *) (from_ptr);           \
        to_type   *__tptr = (to_type *) (to_ptr);           \
        for (i = 0; i < (cnt); i++, __tptr++, __fptr++)         \
            if ((*((unsigned int *)__fptr) & g_QNAN_MASK) != g_QNAN_MASK)   \
                *(__tptr) = (to_type) *(__fptr);        \
            else                            \
                *(__tptr) = (to_type)(0x7FFFFFFFFFFFFFFFLL >> (64 - sizeof (to_type) * 8)); \
    } while (0)

#endif /* __LIBADWIN_MACROS_H__ */
